/*
 ============================================================================
 Project     : Memetic Algorithm
 Author      : Lucas Angelo Silveira
 Universidade de Brasilia (UnB) - Brazil
 Group of Theory of Computation
 Version     : 1.0
 
 Last Modified On: July 16, 2015.
 ============================================================================
 */

#include <stdio.h>
#include <stdlib.h> //malloc, free
#include <string.h> //strcpy
#include <math.h>   //log2
#include "uthash.h"
#include "structure.h"
#include "Calc_entropy.h"


struct mi_struct {
	int id;				// indice usado na hashtable, fitness of a chromosomo
	int nro_casos; 			// numero de casos que ocorre o mesma distancia 
	double probabilidad;    	// probabilidade do genoma id
	UT_hash_handle hh;		// hace que esta estr. sea "hashable
};

struct mi_struct *elementos = NULL;


void add_hash(int id_elto){
	struct mi_struct *s;

	HASH_FIND_INT(elementos, &id_elto, s);  
	if (s==NULL) {
		//Add new chave
		s = (struct mi_struct*)malloc(sizeof(struct mi_struct));
		s->id = id_elto;
		s->nro_casos = 1;
		s->probabilidad = 0;
		HASH_ADD_INT( elementos, id, s ); 
	}
	else { s->nro_casos++;
	}
}


void delete_eltos() {
	struct mi_struct *elto_actual, *tmp;

	HASH_ITER(hh, elementos, elto_actual, tmp) 
	{
		HASH_DEL(elementos, elto_actual);  
		
        free(elto_actual);          
	}
    
}


void show_eltos() {
    struct mi_struct *s;

    for(s=elementos; s != NULL; s=(struct mi_struct*)(s->hh.next)) {
        printf("user id %d: nro casos %d\n", s->id, s->nro_casos);
    }
}

/* Method that computer the entropia of shannon */


float entropia_shannon(population *pob) {
	int i, id, num_eltos;
	struct mi_struct *s;
	float prob, logb2, h, max_h, d;
	for(i=0; i < pob->sizePopulation; i++) {
		id = 0;
		// Add id 
		id = pob->genome[i].fitness;
		add_hash(id);

	}
	// calcular entropia de shannon 
	h=0;
	num_eltos = HASH_COUNT(elementos);
  //  show_eltos();
  //  printf("num_eltos %d\n",num_eltos);
	for(s=elementos; s != NULL; s=(struct mi_struct*)(s->hh.next))
    {
		prob = s->nro_casos * 1.0 / pob->sizePopulation; //definindo pi
		logb2 = (log(prob) / log(2)); //prop. de logaritmos (cambio de base)
		h = h + prob * logb2; // somatorio de prob*log(prob)b2
	}
    h = -1*h; //h: entropia de shannon
  	max_h = log(num_eltos) / log(2); //max entropia de shannon log(x)b2
  	d = h / max_h;// relacao da entropia h y max_h
  	delete_eltos();
	return d;
}





/*
 float entropia_shannon(population *pob) {
 
 int i,j,id, num_eltos;
 int flag=1;
 int counter=1;
 float prob, logb2, h, max_h, d;
 for(i=1; i < pob->sizePopulation; i++) {
 
 if (pob->genome[i].fitness != pob->genome[i-1].fitness) {
 counter++;
 }
 
 }
 
 int *estrut1=(int *) calloc (counter, sizeof (int));
 int *estrut2=(int *) calloc (counter, sizeof (int));
 
 id=0;
 estrut1[id]=pob->genome[0].fitness;
 estrut2[id]=1;
 for (i=1; i<pob->sizePopulation; i++) {
 
 for (j=0; j<counter; j++) {
 if (pob->genome[i].fitness==estrut1[j]) {
 flag=0;
 estrut2[j]+=1;
 break;
 }
 }
 
 if (flag) {
 id++;
 estrut1[id]=pob->genome[i].fitness;
 estrut2[id]=1;
 
 }
 
 flag=1;
 }
 
 
 h=0;
 num_eltos=counter;
 //   printf("Mostrando relacao\n");
 for (i=0; i<counter; i++) {
 //    printf("%d-->%d\n",estrut1[i],estrut2[i]);
 prob=counter *1.0/pob->sizePopulation;
 logb2 = (log(prob) / log(2)); //prop. de logaritmos (cambio de base)
 h = h + prob * logb2; // somatorio de prob*log(prob)b2
 
 }
 h = -1*h; //h: entropia de shannon
 max_h = log(num_eltos) / log(2); //max entropia de shannon log(x)b2
 d = h / max_h;// relacao da entropia h y max_h
 
 
 free(estrut1);
 free(estrut2);
 return d;
 }
 
 
 */






